# Copyright (C) 2006-2008 by Sergio Pistone
#  sergio_pistone@yahoo.com.ar
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the
#  Free Software Foundation, Inc.,
#  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

require File.expand_path( File.dirname( __FILE__ ) + "/../utils/strings" )
require File.expand_path( File.dirname( __FILE__ ) + "/../utils/xmlhash" )
require File.expand_path( File.dirname( __FILE__ ) + "/../i18n/i18n" )
require File.expand_path( File.dirname( __FILE__ ) + "/../gui/gui" )
require File.expand_path( File.dirname( __FILE__ ) + "/pluginadapter" )
require File.expand_path( File.dirname( __FILE__ ) + "/plugins" )
require File.expand_path( File.dirname( __FILE__ ) + "/amarok" )

require "uri"

module WikiPluginAdapter

	# Hack to make module methods become class methods when the module gets included
	def WikiPluginAdapter.included( including )
		if including.is_a?( Class )
			including.extend( ClassMethods ) # adds class methods
		else # including.is_a?( Module )
			including::ClassMethods.append_class_methods( self )
		end
	end

	# Methods under this module will became class methods when the module gets included
	# Note: don't use def self.<method name> but just <method name>
	module ClassMethods
		def ClassMethods.append_class_methods( mod )
			include mod::ClassMethods
		end
	end

	include PluginAdapter

	@@MENU_ENTRY_ALBUM  = I18N.get( "amarok.wikiplugin.checkalbum" )
	@@MENU_ENTRY_SONG   = I18N.get( "amarok.wikiplugin.checksong" )
	@@MENU_UPLOAD_COVER = I18N.get( "amarok.wikiplugin.uploadcover" )

	def submit_check( attempt_session_restore )

 		if ( @submit )
 			if ( attempt_session_restore )
				restore_session_params( @adapter_cookie, @adapter_timestamp, @adapter_username, @adapter_password )
			end
			login( @adapter_username, @adapter_password, false )
		else
			logout()
		end

		if ( ! logged_in?() )
			@submit = false
			@adapter_cookie = ""
			@adapter_timestamp = Time.at( 0 )
		end

		if ( ! @submit || ! @review )
			@prompt_new = false
			@prompt_autogen = false
		end

	end

	def submit?()
		return @submit
	end

	def read_config( config_file )

		values = {
			"#{plugin_name}_submit"			=> nil,
			"#{plugin_name}_review"			=> nil,
			"#{plugin_name}_prompt_autogen"	=> nil,
			"#{plugin_name}_prompt_new"		=> nil,
			"#{plugin_name}_cookie"			=> nil,
			"#{plugin_name}_timestamp"		=> nil,
			"#{plugin_name}_username"		=> nil,
			"#{plugin_name}_password"		=> nil,
		}

		XMLHash.read( config_file, values )

		@review = values["#{plugin_name}_review"] != "false"
		@submit = values["#{plugin_name}_submit"] == "true"
		@prompt_autogen = values["#{plugin_name}_prompt_autogen"] == "true"
		@prompt_new = values["#{plugin_name}_prompt_new"] == "true"
		@adapter_cookie = values["#{plugin_name}_cookie"]
		@adapter_timestamp = Time.at( values["#{plugin_name}_timestamp"].to_i() )
		@adapter_username = Strings.descramble( values["#{plugin_name}_username"] )
		@adapter_password = Strings.descramble( values["#{plugin_name}_password"] )

		submit_check( true )

	end

	def write_config( config_file )

		values = {
			"#{plugin_name}_submit"			=> @submit,
			"#{plugin_name}_review"			=> @review,
			"#{plugin_name}_prompt_autogen"	=> @prompt_autogen,
			"#{plugin_name}_prompt_new"		=> @prompt_new,
			"#{plugin_name}_cookie"			=> @adapter_cookie,
			"#{plugin_name}_timestamp"		=> @adapter_timestamp.to_i(),
			"#{plugin_name}_username"		=> Strings.scramble( @adapter_username ),
			"#{plugin_name}_password"		=> Strings.scramble( @adapter_password ),
		}

		XMLHash.write( config_file, values )

	end

	def configure()

		values = {
			"site_name"			=> site_name(),
			"review"			=> @review,
			"submit"			=> @submit,
			"prompt_autogen"	=> @prompt_autogen,
			"prompt_new"		=> @prompt_new,
			"username"			=> @adapter_username,
			"password"			=> @adapter_password
		}

		if ( GUI.show_wiki_script_config_dialog( values ) )
			@review = values["review"].to_s() != "false"
			@submit = values["submit"].to_s() == "true"
			@prompt_autogen = values["prompt_autogen"].to_s() == "true"
			@prompt_new = values["prompt_new"].to_s() == "true"
			@adapter_username = values["username"]
			@adapter_password = values["password"]
		end

		submit_check( false )

		remove_custom_menu_item( @@MENU_UPLOAD_COVER )
		remove_custom_menu_item( @@MENU_ENTRY_ALBUM )
		remove_custom_menu_item( @@MENU_ENTRY_SONG )
		if ( @submit && logged_in?() )
			add_custom_menu_item( @@MENU_UPLOAD_COVER )
			add_custom_menu_item( @@MENU_ENTRY_ALBUM )
			add_custom_menu_item( @@MENU_ENTRY_SONG )
		end

	end

	def on_start()
		if ( @submit && logged_in?() )
			add_custom_menu_item( @@MENU_UPLOAD_COVER )
			add_custom_menu_item( @@MENU_ENTRY_ALBUM )
			add_custom_menu_item( @@MENU_ENTRY_SONG )
		end
	end

	def on_quit()
		remove_custom_menu_item( @@MENU_UPLOAD_COVER )
		remove_custom_menu_item( @@MENU_ENTRY_ALBUM )
		remove_custom_menu_item( @@MENU_ENTRY_SONG )
	end

	def on_custom_menu_item_selected( menu, item, urls )
		return if ( menu != plugin_name() )
		if ( item == @@MENU_UPLOAD_COVER )
			url = URI.parse( urls[0] )
			upload_album_cover( url.scheme == "file" ? URI.decode( url.path ) : nil)
		elsif ( item == @@MENU_ENTRY_ALBUM )
			url = URI.parse( urls[0] )
			return if ( url.scheme != "file" )
			check_album_page( URI.decode( url.path ) )
		elsif ( item == @@MENU_ENTRY_SONG )
			urls.each() do |url|
				# TODO add skip dialog, show remaining
				url = URI.parse( url )
				return if ( url.scheme != "file" )
				check_song_page( URI.decode( url.path ) )
			end
		end
	end

	def wiki_process_lyrics_data( wiki_checked, artist, title, album, year, ld, notify=false )

		shown = false

		return shown if ( ! submit?() )

		artist = ld["artist"] if ( ! Strings.empty?( ld["artist"] ) )
		title = ld["title"] if ( ! Strings.empty?( ld["title"] ) )
		album = ld["album"] if ( ! Strings.empty?( ld["album"] ) )
		year = ld["year"] if ( ld["year"].to_i() > 1900 )

		if ( ld["site_host"] == site_host() ) # lyrics were found on this site
			if ( ld["custom"]["autogen"] )
				notify( I18N.get( "amarok.wikiplugin.checksong.pagefound.autogenerated", title, artist ) ) if ( notify )
				if ( @prompt_autogen )
					return shown if ( ! submit_authorized?() )
					url, sd = submit_song_page( ld["lyrics"], artist, title, album, year, ld["custom"]["credits"], ld["custom"]["lyricist"], ld["url"], false )
					shown = true
					if ( sd )
						["lyrics", "artist", "title", "year"].each() { |key| ld[key] = sd[key] if ( ! Strings.empty?( sd[key].to_s() ) ) }
						["credits", "lyricst"].each() { |key| ld["custom"][key] = sd[key] if ( ! Strings.empty?( sd[key].to_s() ) ) }
					end
				end
			else
				notify( I18N.get( "amarok.wikiplugin.checksong.pagefound", title, artist ) ) if ( notify )
			end
		else # lyrics were not found on this site (could be that the site wasn't even searched and/or the lyrics were not found)
			begin
				if ( ! wiki_checked && find_song_page_url( artist, title ) )
					notify( I18N.get( "amarok.wikiplugin.checksong.pagefound", title, artist ) ) if ( notify )
					return shown
				end
			rescue TimeoutError
				notify( I18N.get( "amarok.pluginsmanager.search.plugintimeout", plugin_name(), site_host() ) )
				return shown
			end
			if ( @submit && (ld["lyrics"] || @prompt_new) )
				notify( I18N.get( "amarok.wikiplugin.checksong.nopagefound", title, artist ) ) if ( notify )
				return shown if ( ! submit_authorized?() )
				url, sd = submit_song_page( ld["lyrics"], artist, title, album, year, ld["custom"]["credits"], ld["custom"]["lyricist"] )
				shown = true
				if ( sd )
					["lyrics", "artist", "title", "year"].each() { |key| ld[key] = sd[key] if ( ! Strings.empty?( sd[key].to_s() ) ) }
					["credits", "lyricst"].each() { |key| ld["custom"][key] = sd[key] if ( ! Strings.empty?( sd[key].to_s() ) ) }
				end
			end
		end
		return shown
	end


	def check_album_page( file )
		if ( (album_data = Amarok.query_album_data( file )).empty? ) # can't even attempt to search the album...
			notify( I18N.get( "amarok.wikiplugin.checkalbum.noinfofound" ) )
		else
			notify( I18N.get( "amarok.wikiplugin.checkalbum.searchingpage", album_data["album"], album_data["artist"] ) )
			begin
				if ( find_album_page_url( album_data["artist"], album_data["album"], album_data["year"] ) )
					notify( I18N.get( "amarok.wikiplugin.checkalbum.pagefound", album_data["album"], album_data["artist"] ) )
				else
					notify( I18N.get( "amarok.wikiplugin.checkalbum.nopagefound", album_data["album"], album_data["artist"] ) )
					submit_album_page(
						album_data["album"],
						album_data["year"],
						album_data["tracks"],
						album_data["image_path"]
					)
				end
			rescue TimeoutError
				notify( I18N.get( "amarok.pluginsmanager.search.plugintimeout", plugin_name(), site_host() ) )
			end
		end
	end


	def upload_album_cover( file )

		album_data = Amarok.query_album_data( file )
		if ( album_data.empty?() )
			album_data["artist"] = album_data["album"] = album_data["year"] = album_data["image_path"] = ""
		end
		album_data["site_name"] = site_name()

		if ( GUI.show_wiki_upload_cover_dialog( album_data ) )

			album_data["year"] = album_data["year"].to_s()
			if ( Strings.empty?( album_data["artist"] ) || Strings.empty?( album_data["album"] ) ||
				 Strings.empty?( album_data["year"] ) )
				notify( I18N.get( "amarok.wikiplugin.uploadcover.invalidalbumparams" ) )
			elsif ( Strings.empty?( album_data["image_path"] ) )
				notify( I18N.get( "amarok.wikiplugin.uploadcover.noimagepath" ) )
			else
				notify( I18N.get( "amarok.wikiplugin.uploadcover.searching", album_data["album"], album_data["artist"] ) )
				begin
					if ( ! find_album_art_name( album_data["artist"], album_data["album"], album_data["year"] ) )
						upload_cover_image( album_data["image_path"], album_data["artist"], album_data["album"], album_data["year"])
					else
						notify( I18N.get( "amarok.wikiplugin.uploadcover.found" ) )
					end
				rescue TimeoutError
					notify( I18N.get( "amarok.pluginsmanager.search.plugintimeout", plugin_name(), site_host() ) )
				end
			end
		end

	end


	def check_song_page( file )

		song_data = Amarok.query_song_data( file )

		if ( song_data.empty?() )
			notify( I18N.get( "amarok.wikiplugin.checksong.noinfofound" ) )
			return
		end

		artist, title, album, year = song_data["artist"], song_data["title"], song_data["album"], song_data["year"]

		notify( I18N.get( "amarok.wikiplugin.checksong.searchingpage", title, artist ) )
		lyrics_data = self.lyrics_full_search( artist, title, album, year )

		if ( ! lyrics_data["lyrics"] )

			# first we try to find the lyrics in Amarok database
			lyrics_data = normalize_lyrics_data( {}, artist, title, album, year )
			lyrics_data["site_host"] = ""
			lyrics_data["lyrics"] = Amarok.query_song_lyrics( file )

			# if we didn't found them there, we try the other selected sites
			if ( ! lyrics_data["lyrics"] )
				used_plugins = Plugins.used_plugins().clone()
				used_plugins.delete( self )
				used_plugins.each() do |plugin|
					begin
						lyrics_data = plugin.lyrics_full_search( artist, title, album, year )
						break if ( lyrics_data["lyrics"] )
					rescue TimeoutError
						notify( I18N.get( "amarok.pluginsmanager.search.plugintimeout",
								plugin.plugin_name(), plugin.site_host() ) )
					end
				end
			end
		end

		wiki_process_lyrics_data( true, artist, title, album, year, lyrics_data, true )

	end

end
