#!/usr/bin/ruby

# Copyright (C) 2006-2008 by Sergio Pistone
#  sergio_pistone@yahoo.com.ar
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the
#  Free Software Foundation, Inc.,
#  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

require File.expand_path( File.dirname( __FILE__ ) + "/../i18n/i18n" )
require File.expand_path( File.dirname( __FILE__ ) + "/../utils/strings" )
require File.expand_path( File.dirname( __FILE__ ) + "/optionsparser" )
require File.expand_path( File.dirname( __FILE__ ) + "/plugins" )

options = OptionsParser.parse( ARGV )

Plugins.load_plugins( options.sites )
Plugins.all_plugins().each() { |plugin| plugin.log_file = "#{ENV["HOME"]}/.wikilyrics.log" }


class PluginsManager

	def initialize( featuring_fix, cleanup_lyrics, submit, review=true,
					prompt_autogen=false, prompt_new=false, username=nil, password=nil )
		@featuring_fix = featuring_fix
		@submit_plugin = nil
		Plugins.all_plugins().each() do |plugin|
			plugin.cleanup_lyrics = cleanup_lyrics
			if ( plugin.plugin_name() == submit )
				@submit_plugin = plugin
				@submit_plugin.set_submit_settings( username, password, review, prompt_autogen, prompt_new )
			end
		end
	end

	def PluginsManager.notify( message )
		puts "Wiki-Lyrics: " + message.gsub( /\/?<[^>]+>/, "" )
	end

	def notify( message )
		self.class.notify( message )
	end

	def restore_session( session_file )
		@submit_plugin.restore_session( session_file ) if ( @submit_plugin )
	end

	def save_session( session_file )
		@submit_plugin.save_session( session_file ) if ( @submit_plugin )
	end

	def lyrics_full_search( artist, title, album, year )
		shown = false
		wiki_names = Plugins.wiki_names()
		searched_wikis = []
		suggestions = []
		Plugins.used_plugins().each() do |plugin|
			begin
				lyrics_data = plugin.lyrics_full_search( artist, title, album, year )
				if ( lyrics_data["suggestions"].size > 0 )
					suggestions << plugin.plugin_name()
					suggestions.concat( lyrics_data["suggestions"] )
					lyrics_data["suggestions"] = suggestions
				end
				searched_wikis << plugin.plugin_name() if ( wiki_names.include?( plugin.plugin_name() ) )
 				if ( lyrics_data["lyrics"] )
					shown = wiki_process_lyrics_data( searched_wikis, artist, title, album, year, lyrics_data )
					return lyrics_data, shown
				end
			rescue TimeoutError
				notify( I18N.get( "cli.pluginsmanager.plugintimeout", plugin.plugin_name(), plugin.site_host() ) )
			end
		end
		lyrics_data = Lyrics.normalize_lyrics_data( { "suggestions" => suggestions }, artist, title, album, year )
		shown = wiki_process_lyrics_data( searched_wikis, artist, title, album, year, lyrics_data )
		return lyrics_data, shown
	end

	def wiki_process_lyrics_data( searched_wikis, artist, title, album, year, lyrics_data )
		shown = false
		Plugins.wiki_plugins().each() do |plugin|
			next if ( ! plugin.submit?() )
			shown = true if ( plugin.wiki_process_lyrics_data(
				searched_wikis.include?( plugin.plugin_name() ),
				artist,
				title,
				album,
				year,
				lyrics_data
			) )
		end
		return shown
	end

	def process( artist, title, album, year )

		if ( @featuring_fix )
			artist = Strings.cleanup_artist( artist, title )
			title  = Strings.cleanup_title( title )
		end

		notify( I18N.get( "cli.pluginsmanager.searchinglyrics", title, artist ) )

		lyrics_data, shown = lyrics_full_search( artist, title, album, year )

		if ( lyrics_data["lyrics"] )
			notify( I18N.get( "cli.pluginsmanager.lyricsfound", lyrics_data["title"], lyrics_data["artist"], lyrics_data["site_name"] ) )
			if ( ! shown )
				if ( GUI.get_preferred_toolkit() )
					GUI.show_lyrics_dialog( lyrics_data )
				else
					puts "\n#{lyrics_data["lyrics"]}\n\n"
				end
			end
		else
			notify( I18N.get( "cli.pluginsmanager.nolyricsfound", lyrics_data["title"], lyrics_data["artist"] ) )
		end

		return lyrics_data

	end

end

HTTP.set_proxy_settings( options.proxy ) if ( options.proxy )
GUI.set_toolkit_priority( options.toolkits ) if ( options.toolkits.size() > 0 )

lyrics_manager = PluginsManager.new(
	options.feat_fix,
	options.cleanup,
	options.submit,
	options.review,
	options.prompt_autogen,
	options.prompt_new,
	options.user,
	options.pass
)

lyrics_manager.restore_session( options.session_file ) if ( options.session_file )

Plugins.used_plugins().each() { |a| puts a }

if ( options.batch_file )
	notify( I18N.get( "cli.pluginsmanager.batchmode" ) )
	file = File.new( options.batch_file, "r" )
	count = 0
	while ( line = file.gets )
		params = line.split( ";" )
		next if ( params.length < 2 )
		ld = lyrics_manager.process( params[0], params[1], params[2], params[3] )
		count += 1
	end
	notify( I18N.get( "cli.pluginsmanager.processedcount", count ) )
	file.close()
else
	ld = lyrics_manager.process( options.artist, options.title, options.album, options.year )
end

lyrics_manager.save_session( options.session_file ) if ( options.session_file )
