# Copyright (C) 2006-2008 by Sergio Pistone
#  sergio_pistone@yahoo.com.ar
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the
#  Free Software Foundation, Inc.,
#  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

require File.expand_path( File.dirname( __FILE__ ) + "/../utils/strings" )
require File.expand_path( File.dirname( __FILE__ ) + "/../i18n/i18n" )
require File.expand_path( File.dirname( __FILE__ ) + "/../gui/gui" )
require File.expand_path( File.dirname( __FILE__ ) + "/pluginadapter" )
require File.expand_path( File.dirname( __FILE__ ) + "/plugins" )

require "md5"
require "uri"

module WikiPluginAdapter

	# Hack to make module methods become class methods when the module gets included
	def WikiPluginAdapter.included( including )
		if including.is_a?( Class )
			including.extend( ClassMethods ) # adds class methods
		else # including.is_a?( Module )
			including::ClassMethods.append_class_methods( self )
		end
	end

	# Methods under this module will became class methods when the module gets included
	# Note: don't use def self.<method name> but just <method name>
	module ClassMethods
		def ClassMethods.append_class_methods( mod )
			include mod::ClassMethods
		end
	end

	include PluginAdapter

	def submit?()
		return @submit
	end

	def set_submit_settings( username, password, review, prompt_autogen, prompt_new )
		@submit = true
		@review = review
		@prompt_autogen = prompt_autogen
		@prompt_new = prompt_new
		@username = username
		@password = password
		if ( ! @submit || ! @review )
			@prompt_new = false
			@prompt_autogen = false
		end
	end

	def wiki_process_lyrics_data( site_checked, artist, title, album, year, ld )

		shown = false
		artist = ld["artist"] if ( ! Strings.empty?( ld["artist"] ) )
		title = ld["title"] if ( ! Strings.empty?( ld["title"] ) )
		album = ld["album"] if ( ! Strings.empty?( ld["album"] ) )
		year = ld["year"] if ( ld["year"].to_i() > 1900 )

		if ( ld["site_host"] == site_host() ) # lyrics were found on this site
			if ( ld["custom"]["autogen"] )
				if ( @prompt_autogen )
					url, sub_data = submit_song_page( ld["lyrics"], artist, title, album, year, ld["custom"]["credits"], ld["custom"]["lyricist"], ld["url"] )
					shown = true
					if ( sub_data )
						["lyrics", "artist", "title", "year", "credits", "lyricst"].each() do |key|
							ld[key] = sub_data[key] if ( Strings.empty?( sub_data[key].to_s() ) )
						end
					end
				end
			end
		else # lyrics were not found on this site (could be that the site wasn't even searched and/or the lyrics were not found)
			begin
				return if ( ! site_checked && find_song_page_url( artist, title ) )
			rescue TimeoutError
				notify( I18N.get( "amarok.pluginsmanager.search.plugintimeout", plugin_name(), site_host() ) )
				return
			end
			if ( @submit && ( ld["lyrics"] || @prompt_new ) )
				url, sub_data = submit_song_page( ld["lyrics"], artist, title, album, year, ld["custom"]["credits"], ld["custom"]["lyricist"] )
				shown = true
				if ( sub_data )
					["lyrics", "artist", "title", "year", "credits", "lyricst"].each() do |key|
						ld[key] = sub_data[key] if ( Strings.empty?( sub_data[key].to_s() ) )
					end
				end
			end
		end
		return shown
	end

end
