# Copyright (C) 2006-2008 by Sergio Pistone
#  sergio_pistone@yahoo.com.ar
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the
#  Free Software Foundation, Inc.,
#  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

module GUI

	@@toolkits = []

	def GUI.set_toolkit_priority( toolkits )
		toolkits.each() do |toolkit|
			if ( @@toolkits.include?( toolkit ) )
				# check if the toolkit is already loaded (and change it's priority)
				@@toolkits.delete( toolkit )
				@@toolkits.insert( 0, toolkit )
				return toolkit
			elsif ( GUI.load_toolkit( toolkit ) ) # try to load the toolkit
				@@toolkits.insert( 0, toolkit )
				return toolkit
			end
		end
		return GUI.get_preferred_toolkit()
	end

	def GUI.load_toolkit( toolkit )
		toolkit = toolkit.downcase()
		if ( toolkit == "qt" || toolkit == "qt4" )
			begin
				return require( File.expand_path( File.dirname( __FILE__ ) + "/gui-qt4" ) )
			rescue LoadError => e
				$stderr << "Error loading Qt4 GUI backend: #{e}\n"
			end
		elsif ( toolkit == "qt3" )
			begin
				return require( File.expand_path( File.dirname( __FILE__ ) + "/gui-qt3" ) )
			rescue LoadError => e
				$stderr << "Error loading Qt3 GUI backend: #{e}\n"
			end
		elsif ( toolkit == "gtk" )
			begin
				return require( File.expand_path( File.dirname( __FILE__ ) + "/gui-gtk" ) )
			rescue LoadError => e
				$stderr << "Error loading GTK GUI backend: #{e}\n"
			end
		elsif ( toolkit == "tk" )
			begin
				return require( File.expand_path( File.dirname( __FILE__ ) + "/gui-tk" ) )
			rescue LoadError => e
				$stderr << "Error loading Tk GUI backend: #{e}\n"
			end
		end
		return false
	end

	def GUI.get_preferred_toolkit()
		return @@toolkits.length > 0 ? @@toolkits[0].downcase() : nil
	end

	def GUI.show_dialog( dlg_construct, *args )

		pref_toolkit = GUI.get_preferred_toolkit()

		if ( pref_toolkit == "qt" || pref_toolkit == "qt4" )
			app = Qt::Application.new( ARGV )
			dialog = eval( "QT4::" + dlg_construct )
			dialog.show()
			app.exec()
		elsif ( pref_toolkit == "qt3" )
			app = Qt::Application.new( ARGV )
			dialog = eval( "QT3::" + dlg_construct )
			app.setMainWidget( dialog )
			app.mainWidget.show()
			app.exec()
		elsif ( pref_toolkit == "gtk" )
			dialog = eval( "GTK::" + dlg_construct )
			dialog.exec()
			Gtk.main()
		elsif ( pref_toolkit == "tk" )
			dialog = eval( "TK::" + dlg_construct )
			dialog.exec()
		end

		if ( pref_toolkit == nil )
			return nil
		elsif dialog.accepted()
			return dialog.values()
		else
			return nil
		end

	end

	def GUI.show_manager_script_config_dialog( values )
		values["script_name"] = "WikiLyrics" if ( ! values.include?( "script_name" ) )
		ret = GUI.show_dialog( "PluginsManagerConfigDialog.new( args[0] )", values )
		values.update( ret ) if ( ret )
		return ret != nil
	end

	def GUI.show_wiki_script_config_dialog( values )
		values["site_name"] = "WikiLyrics" if ( ! values.include?( "site_name" ) )
		ret = GUI.show_dialog( "WikiPluginConfigDialog.new( args[0] )", values )
		values.update( ret ) if ( ret )
		values["submit"] = values["submit"].to_s() == "true"
		values["review"] = values["review"].to_s() == "true"
		values["prompt_autogen"] = values["prompt_autogen"].to_s() == "true"
		values["prompt_new"] = values["prompt_new"].to_s() == "true"
		return ret != nil
	end

	def GUI.show_wiki_submit_song_dialog( values )
		values["year"] = values["year"].to_i()
		values["site_name"] = "WikiLyrics" if ( ! values.include?( "site_name" ) )
		ret = GUI.show_dialog( "WikiSubmitSongDialog.new( args[0] )", values )
		values.update( ret ) if ( ret )
		if ( values["instrumental"] )
			values["lyrics"] = nil
		else
			values["lyrics"] = values["lyrics"].to_s().strip()
		end
		values["autogen"] = values["autogen"].to_s() == "true"
		values["instrumental"] = values["instrumental"].to_s() == "true"
		return ret != nil
	end

	def GUI.show_wiki_submit_album_dialog( values )
		values["released"] = values["released"].to_s()
		values["site_name"] = "WikiLyrics" if ( ! values.include?( "site_name" ) )
		ret = GUI.show_dialog( "WikiSubmitAlbumDialog.new( args[0] )", values )
		values.update( ret ) if ( ret )
		values["autogen"] = values["autogen"].to_s() == "true"
		return ret != nil
	end

	def GUI.show_wiki_upload_cover_dialog( values )
		values["site_name"] = "WikiLyrics" if ( ! values.include?( "site_name" ) )
		values["year"] = values["year"].to_i()
		ret = GUI.show_dialog( "WikiUploadCoverDialog.new( args[0] )", values )
		values.update( ret ) if ( ret )
		return ret != nil
	end

	def GUI.show_wiki_fix_pages_dialog( values )
		values["title"] = "Pages fixer" if ( ! values.include?( "title" ) )
		GUI.show_dialog( "WikiFixPagesDialog.new( args[0] )", values )
	end

	def GUI.show_search_lyrics_dialog( values )
		values["year"] = values["year"].to_i()
		ret = GUI.show_dialog( "SearchLyricsDialog.new( args[0] )", values )
		values.update( ret ) if ( ret )
		return ret != nil
	end

	def GUI.show_lyrics_dialog( lyrics )
		GUI.show_dialog( "LyricsDialog.new( args[0] )", lyrics )
	end

	def GUI.show_message_dialog( message, title=nil )
		if ( title )
			system( "kdialog", "--icon", "amarok", "--title", title, "--msgbox", "<qt>" + message.gsub( "\n", "<br/>" ) + "</qt>" )
		else
			system( "kdialog", "--icon", "amarok", "--msgbox", "<qt>" + message.gsub( "\n", "<br/>" ) + "</qt>" )
		end
	end

	def GUI.show_confirmation_dialog( message, title=nil )
		if ( title )
			system( "kdialog", "--icon", "amarok", "--title", title, "--yesno", "<qt>" + message.gsub( "\n", "<br/>" ) + "</qt>" )
		else
			system( "kdialog", "--icon", "amarok", "--yesno", "<qt>" + message.gsub( "\n", "<br/>" ) + "</qt>" )
		end
		return $? == 0
	end

end

if ( $0 == __FILE__ )
	# NOTE: qt4 can't be loaded along with qt3 or gtk (no idea why...)
#	["qt4"].each() do |toolkit|
	["qt3", "gtk", "tk"].each() do |toolkit|

		GUI.set_toolkit_priority( toolkit )

		values = { "script_name"=>"script_name", "used_plugins"=>["used1", "used2", "used3", "used4"], "unused_plugins"=>["unused1", "unused2"], "cleanup_lyrics"=>"true", "write_log"=>"true" }
		puts GUI.show_manager_script_config_dialog( values )
		values.each() { |key, val| puts "#{key}: #{val}" }

		values = { "artist"=>"artist", "title"=>"title", "lyrics"=>"lyrícs1\nlyrícs2", "site"=>"site" }
		puts GUI.show_lyrics_dialog( values )
		values.each() { |key, val| puts "#{key}: #{val}" }

		values = { "artist"=>"artist", "title"=>"title", "album"=>"album", "year"=>"1969" }
		puts GUI.show_search_lyrics_dialog( values )
		values.each() { |key, val| puts "#{key}: #{val}" }

		values = { "site_name"=>"site_name", "submit"=>"true", "review"=>"true", "prompt_autogen"=>"false", "prompt_new"=>"false", "username"=>"username", "password"=>"password" }
		puts GUI.show_wiki_script_config_dialog( values )
		values.each() { |key, val| puts "#{key}: #{val}" }

		values = { "site_name"=>"site_name", "url"=>"url", "artist"=>"artist", "year"=>"1999", "album"=>"album", "title"=>"title", "lyrics"=>"lyrícs1\nlyrícs2", "lyricist"=>"lyricist", "credits"=>"credits", "autogen"=>"false" }
		puts GUI.show_wiki_submit_song_dialog( values )
		values.each() { |key, val| puts "#{key}: #{val}" }

		values = { "site_name"=>"site_name", "artist"=>"artist", "year"=>"1999", "month"=>"10", "day"=>"14","album"=>"album", "autogen"=>"false", "image_path"=>"image_path", "tracks"=>"tracks <small>fffff</small>\ntracks <small>fffff</small>" }
		puts GUI.show_wiki_submit_album_dialog( values )
		values.each() { |key, val| puts "#{key}: #{val}" }

		values = { "site_name"=>"site_name", "artist"=>"artist", "year"=>"1999", "month"=>"10", "day"=>"14","album"=>"album", "autogen"=>"false", "tracks"=>"tracks <small>fffff</small>\ntracks <small>fffff</small>" }
		puts GUI.show_wiki_submit_album_dialog( values )
		values.each() { |key, val| puts "#{key}: #{val}" }

		GUI.show_wiki_fix_pages_dialog( {
			"site_name" => "site_name",
			"title" => "Pages fixer",
			"fetch_func" => proc { `kdialog --msgbox 'fetch_func'` },
			"fix_func" => proc { `kdialog --msgbox 'fix_func'` },
			"submit_func" => proc { `kdialog --msgbox 'submit_func'` }
		} )

		values = { "site_name"=>"site_name", "artist"=>"artist", "album"=>"album", "year"=>"year", "image_path" => "image_path" }
		puts GUI.show_wiki_upload_cover_dialog( values )
		values.each() { |key, val| puts "#{key}: #{val}" }

	end
end
