# Copyright (C) 2006-2008 by Sergio Pistone
#  sergio_pistone@yahoo.com.ar
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the
#  Free Software Foundation, Inc.,
#  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

$LOAD_PATH << File.expand_path(File.dirname(__FILE__))

require "utils/strings"
require "lyrics"

class AZLyrics < Lyrics

	def AZLyrics.site_host()
		return "www.azlyrics.com"
	end

	def AZLyrics.site_name()
		return "AZ Lyrics"
	end

	def AZLyrics.get_test_data()
		return [
			{ "artist"=>"Radiohead", "album"=>"Kid A", "title"=>"Optimistic" },
			{ "artist"=>"Pearl Jam", "album"=>"Ten", "title"=>"Alive" },
			{ "artist"=>"Massive Attack", "album"=>"Protection", "title"=>"Protection" },
			{ "artist"=>"Portishead", "album"=>"Dummy", "title"=>"Wandering Star" },
		]
	end

	def AZLyrics.build_song_add_url( artist, title, album=nil, year=nil )
		return "http://#{site_host()}/add.html"
	end

	def AZLyrics.build_google_feeling_lucky_url( artist, title=nil )
		query = Strings.google_search_quote( artist )
		query << " " << Strings.google_search_quote( title ) if ( title )
		query << " lyrics"
		return Strings.build_google_feeling_lucky_url( query, title ? "#{site_host()}/lyrics" : site_host() )
	end

	def build_lyrics_fetch_data( artist, title, album=nil, year=nil )
		return { "url" => build_google_feeling_lucky_url( artist, title ) }
	end

	def check_lyrics_page( page_body, artist, title, album=nil, year=nil )
		md = /<TITLE>([^<]+) LYRICS - ([^<]+)<\/TITLE>/.match( page_body )
		return md ?
			Strings.normalize( artist ) == Strings.normalize( md[1] ) &&
			Strings.normalize( title ) == Strings.normalize( md[2] ) :
			false
	end

	def parse_lyrics( url, body, artist, title, album=nil, year=nil )

		body = Strings.latin12utf8( body )
		body.tr_s!( " \n\r\t", " " )

		lyrics_data = {}

		if ( md = /<B>([^<]+) LYRICS ?<\/B> ?<BR> ?<BR> ?<FONT size=2> ?<B> ?"([^<]+)" ?<\/b>/.match( body ) )
			lyrics_data["artist"], lyrics_data["title"] = Strings.titlecase( md[1], true, true ), md[2]
		end

		return lyrics_data if ( ! body.sub!( /^.*"<\/b><BR> ?<BR> ?/i, "" ) )
		return lyrics_data if ( ! body.sub!( /\ ?<BR> ?<BR> ?\[ <a href=.*$/i, "" ) )

		body.gsub!( /<i>\[Thanks to.*$/i, "" )
		body.gsub!( /\ ?<br ?\/?> ?/i, "\n" )
		body.gsub!( /\n{3,}/, "\n\n" )

		lyrics_data["lyrics"] = body

		return lyrics_data
	end

	def build_suggestions_fetch_data( artist, title, album=nil, year=nil )
		return { "url" => build_google_feeling_lucky_url( artist ) }
	end

	def check_suggestions_page( page_body, artist, title, album=nil, year=nil )
		md = /<TITLE>([^<]+) lyrics<\/TITLE>/.match( page_body )
		return md ? Strings.normalize( artist ) == Strings.normalize( md[1] ) : false
	end

	def parse_suggestions( url, body, artist, title, album=nil, year=nil )

		body = Strings.latin12utf8( body )
		body.tr_s!( " \n\r\t", " " )

		suggestions = []

		return suggestions if ( ! body.gsub!( /.*<tr><td align=center valign=top> <font face=verdana size=5><br> ?<b>[^<]+ lyrics<\/b>/i, "" ) )
		return suggestions if ( ! body.gsub!( /<\/font> ?<\/font> ?<\/td> ?<\/tr> ?<\/table>.*$/i, "" ) )

		body.split( /<br>/i ).each() do |entry|
			if ( md = /<a href="\.\.([^"]+)" target="_blank">([^<]+)<\/a>/i.match( entry ) )
				suggestions << { "url"=>"http://#{site_host()}#{md[1]}", "artist"=>artist, "title"=>md[2] }
			end
		end

		return suggestions
	end

end
