# Copyright (C) 2006-2008 by Sergio Pistone
#  sergio_pistone@yahoo.com.ar
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the
#  Free Software Foundation, Inc.,
#  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

$LOAD_PATH << File.expand_path(File.dirname(__FILE__))

require "utils/strings"
require "lyrics"

class DarkLyrics < Lyrics

	def DarkLyrics.site_host()
		return "www.darklyrics.com"
	end

	def DarkLyrics.site_name()
		return "Dark Lyrics"
	end

	def DarkLyrics.get_test_data()
		return [
			{ "artist"=>"Opeth", "album"=>"Damnation", "title"=>"Hope Leaves" },
			{ "artist"=>"Megadeth", "album"=>"Youthanasia", "title"=>"À tout le monde" },
			{ "artist"=>"Metallica", "album"=>"Master of Puppets", "title"=>"Master of Puppets" },
			{ "artist"=>"Lacuna Coil", "album"=>"Comalies", "title"=>"Heaven's a Lie" },
		]
	end

	def DarkLyrics.build_song_add_url( artist, title, album=nil, year=nil )
		return "http://#{site_host()}/submit.html"
	end

	def DarkLyrics.build_google_feeling_lucky_url( artist, title=nil )
		query = Strings.google_search_quote( artist )
		query << " " << Strings.google_search_quote( title ) if ( title )
		query << " lyrics"
		return Strings.build_google_feeling_lucky_url( query, title ? "#{site_host()}/lyrics" : site_host() )
	end

	def build_lyrics_fetch_data( artist, title, album=nil, year=nil )
		return { "url" => build_google_feeling_lucky_url( artist, title ) }
	end

	def check_lyrics_page( page_body, artist, title, album=nil, year=nil )
		md = /<title>([^<]+) LYRICS - [^<]+<\/title>/.match( page_body )
		return md ? Strings.normalize( artist ) == Strings.normalize( md[1] ) : false
	end

	def parse_lyrics( url, body, artist, title, album=nil, year=nil )

		body = Strings.latin12utf8( body )
		body.tr_s!( " \n\r\t", " " )

		lyrics_data = {}

		if ( md = /<FONT size=5 color=#FFFFCC>([^<]+) LYRICS<\/FONT><br>/.match( body ) )
			lyrics_data["artist"] = Strings.titlecase( md[1], true, true )
		end

		if ( md = /<FONT size=3 color=white><b>([^<]+) \(([0-9]{4,4})( [^\)]+|)\)<\/b><\/FONT><br>/.match( body ) )
			lyrics_data["album"], lyrics_data["year"] = md[1], md[2]
		end

		return lyrics_data if ( ! body.sub!( /^.*<SCRIPT LANGUAGE="javascript" src="\.\.\/\.\.\/recban\.js"><\/SCRIPT><BR>/, "" ) )
		return lyrics_data if ( ! body.sub!( /<FONT [^>]*size=.*$/i, "" ) )

		md = /.*#([0-9]+)$/.match( url.to_s() )
		track = md ? md[1] : nil
		normalized_title = Strings.normalize( title )
		body.split( /<a name=[0-9]+><FONT color=#DDDDDD>/i ).each() do |song_content|
			md = /^<b>([0-9]+)\. ([^<]+)<\/b><\/font><br>(.*)$/.match( song_content )
			next if ( ! md || ( track && track != md[1] ) || ( ! track && normalized_title != Strings.normalize( md[2] ) ) )
			lyrics_data["title"] = md[2]
			lyrics_data["lyrics"] = md[3]
			lyrics_data["lyrics"].gsub!( /\ ?<br ?\/?> ?/i, "\n" )
			lyrics_data["lyrics"].gsub!( /\n{3,}/, "\n\n" )
			lyrics_data["lyrics"].strip!()
			return lyrics_data
		end

		return lyrics_data
	end

	def build_suggestions_fetch_data( artist, title, album=nil, year=nil )
		return { "url" => build_google_feeling_lucky_url( artist ) }
	end

	def check_suggestions_page( page_body, artist, title, album=nil, year=nil )
		md = /<title>([^<]+) LYRICS<\/title>/.match( page_body )
		return md ? Strings.normalize( artist ) == Strings.normalize( md[1] ) : false
	end

	def parse_suggestions( url, body, artist, title, album=nil, year=nil )

		body = Strings.latin12utf8( body )
		body.tr_s!( " \n\r\t", " " )

		suggestions = []

		return suggestions if ( ! body.sub!( /^.*<FONT FACE="Helvetica" COLOR=#FFFFCC SIZE=4><br>[^<]+ LYRICS<BR><\/FONT>/i, "" ) )
		return suggestions if ( ! body.sub!( /<SCRIPT LANGUAGE="javascript".*$/i, "" ) )

		body.split( /<br>/i ).each() do |entry|
			if ( md = /<a href="\.\.(\/lyrics\/[^"]+)" target="_blank"><FONT [^>]+>([^"]+)<\/FONT><\/a>/i.match( entry ) )
				suggestions << { "url"=>"http://#{site_host()}#{md[1]}", "artist"=>artist, "title"=>md[2] }
			end
		end

		return suggestions
	end

end
