#  Copyright (C) 2007-2008 by
#  Swapan Sarkar <swapan@yahoo.com>
#  Sergio Pistone <sergio_pistone@yahoo.com.ar>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the
#  Free Software Foundation, Inc.,
#  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

$LOAD_PATH << File.expand_path(File.dirname(__FILE__))

require "utils/strings"
require "utils/itrans"
require "lyrics"

require "cgi"

class Giitaayan < Lyrics

# 	@@prefix = "http://s94437128.onlinehome.us/isb/cisb/"
# 	@@prefix = "http://gaane.giitaayan.com/cisb/"
	@@prefix = "http://thaxi.hsc.usc.edu/rmim/giitaayan/cisb/"
	@@prefix_regexp = /<a href=" *(#{@@prefix.gsub( "\\", "\\\\" ).gsub( ".", "\\." )}[^"]+.isb)">/

	def Giitaayan.site_host()
		return "www.giitaayan.com"
	end

	def Giitaayan.site_name()
		return "Giitaayan"
	end

	def Giitaayan.get_test_data()
		return [
			{ "artist"=>"Abhijit, Kavita Subramaniam", "album"=>"Juhi", "title"=>ITRANS.to_devanagari( "a.ndherii raat me.n ham aur tum" ) },
			{ "artist"=>"Poornima, Chorus", "album"=>"Raajaa Baabu", "title"=>ITRANS.to_devanagari( "a aa ii u u uu meraa dil na to.Do" ) },
			{ "artist"=>"Sonu Nigam, Chorus", "album"=>"Fizaa", "title"=>ITRANS.to_devanagari("gayaa gayaa dil jaane de" ) },
			{ "artist"=>"Lata, Hemant", "album"=>"Ek Saal", "title"=>ITRANS.to_devanagari( "ulajh gaye do nainaa, dekho" ) },
		]
	end

	def Giitaayan.known_url?( url )
		return url.index( @@prefix ) == 0
	end

	def Giitaayan.build_song_add_url( artist, title, album=nil, year=nil )
		return "http://#{site_host()}/submit.asp"
	end

	def parse_lyrics( url, body, artist, title, album=nil, year=nil )

		lyrics_data = { "custom" => {} }

		["stitle", "film", "year", "starring", "singer", "music", "lyrics"].each() do |key|
			if ( md = /\\#{key}\{(.+)}%/.match( body ) )
				lyrics_data["custom"][key] = md[1]
			end
		end

		lyrics_data["artist"] = lyrics_data["custom"]["singer"] if ( ! Strings.empty?( lyrics_data["custom"]["singer"] ) )
		lyrics_data["album"] = lyrics_data["custom"]["film"] if ( ! Strings.empty?( lyrics_data["custom"]["film"] ) )
		lyrics_data["year"] = lyrics_data["custom"]["year"] if ( ! Strings.empty?( lyrics_data["custom"]["year"] ) )
		lyrics_data["lyricist"] = lyrics_data["custom"]["lyricist"] if ( ! Strings.empty?( lyrics_data["custom"]["lyrics"] ) )
		if ( ! Strings.empty?( lyrics_data["custom"]["music"] ) || ! Strings.empty?( lyrics_data["custom"]["lyrics"] ) )
			lyrics_data["credits"] = "#{lyrics_data["custom"]["music"]} #{lyrics_data["custom"]["lyrics"]}".strip()
		end
		if ( ! Strings.empty?( lyrics_data["custom"]["stitle"] ) )
			lyrics_data["custom"]["stitle"] = ITRANS.to_devanagari( lyrics_data["custom"]["stitle"] )
			lyrics_data["title"] = lyrics_data["custom"]["stitle"]
		end

		return lyrics_data if ( ! body.gsub!( /^.*\n#indian\s*\n%?/m, "" ) )
		return lyrics_data if ( ! body.gsub!( /%?\s*\n#endindian.*$/m, "" ) )

		body.gsub!( "\\threedots", "..." )
		body.gsub!( "\\-", "-" )
		body.gsub!( "\\:", ":" )
		body.gsub!( /%[^\n]*/, "" )

		lyrics_data["lyrics"] = ITRANS.to_devanagari( body )

		return lyrics_data
	end

	def build_suggestions_fetch_data( artist, title, album=nil, year=nil )
		return { "url" => "http://#{site_host()}/search.asp?browse=stitle&s=#{CGI.escape(ITRANS.from_devanagari( title ))}" }
	end

	# returns an array of maps with following keys: url, artist, title
	def parse_suggestions( url, body, artist, title, album=nil, year=nil )

		body.tr_s!( " \n\r\t", " " )

		suggestions = []

		return suggestions if ( body.include?( "Sorry, no song found for your search!" ) )
		return suggestions if ( ! body.sub!( /^.*<div align="center"><b>Lyrics<\/b><\/div> ?<\/td> ?<\/tr> ?<tr>/, "" ) )
		return suggestions if ( ! body.sub!( /<form name="form1" method="get" action="search\.asp".*$/, "" ) )

		body.gsub!( /<br> ?Page <b>1<\/b> ?<a href="search.asp\?PageNo=2&s=na&browse=stitle">.*$/, "" )

		body.split( /<\/tr> ?<tr> ?/ ).each() do |sugg|
			if ( ( md1 = /^ ?<td>([^<]+) - <a/.match( sugg ) ) &&
				 ( md2 = @@prefix_regexp.match( sugg ) ) )
				suggestions << { "url"=>md2[1], "artist"=>artist, "title"=>ITRANS.to_devanagari( md1[1] ) }
			end
		end
		return suggestions
	end

end
