#  Copyright (C) 2006-2008 by
#  Eduardo Robles Elvira <edulix@gmail.com>
#  Sergio Pistone <sergio_pistone@yahoo.com.ar>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the
#  Free Software Foundation, Inc.,
#  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

$LOAD_PATH << File.expand_path(File.dirname(__FILE__))

require "utils/strings"
require "utils/http"
require "lyrics"

class Jamendo < Lyrics

	def Jamendo.site_host()
		return "www.jamendo.com"
	end

	def Jamendo.site_name()
		return "Jamendo"
	end

	def Jamendo.get_test_data()
		return [
			{ "artist"=>"Misantropía", "album"=>"Misantropía", "title"=>"Hipócrita (Hypocrite)" },
			{ "artist"=>"Inesperado", "album"=>"Maqueta Inesperada", "title"=>"Pez Gordo" },
			{ "artist"=>"Tirpunk", "album"=>"Tirpunk-Újra", "title"=>"Suicide" },
			{ "artist"=>"Punkamine", "album"=>"Punkamine", "title"=>"Gora gerra" },
		]
	end

	def Jamendo.cleanup_artist_name( artist )
		artist = Strings.downcase( artist )
		Strings.remove_vocal_accents!( artist )
		artist.gsub!( /\[|\]|:/, "" )
		artist.gsub!( /_| |'|"|ø|&|@|\/|\*|\.|®|%|#/, " " ) # \' can also be ''
		artist.squeeze!( " " )
		artist.strip!()
		artist.gsub!( " ", "." )
		return artist
	end

	def cleanup_artist_name( artist )
		self.class.cleanup_artist_name( artist )
	end

	def suggestions( artist, title, album=nil, year=nil )

		suggestions = []

		artist_url = "http://#{site_host()}/en/artist/#{cleanup_artist_name( artist )}/"

		if ( log?() )
			log( site_name().upcase() )
			log( "Retrieving SUGGESTIONS..." )
			log( " - received artist: #{artist}" )
			log( " - received title: #{title}" )
			log( " - received album: #{album}" ) if ( album )
			log( " - received year: #{year}" ) if ( year )
			log( " - built artist url: #{artist_url}" )
			log( "Fetching artist page... ", 0 )
		end

		response, artist_url = HTTP.fetch_page_get( artist_url, nil, 0 )

		if ( response && response.body() )
			error = response.body().include?( "has no album on <b>jamendo</b> yet" )
			if ( error && log?() )
				log( "INVALID PAGE" )
			elsif ( log?() )
				log( "OK" )
			end
			log( response.body(), 2 ) if ( long_log?() )
		else
			error = true
			log( "ERROR" ) if ( log?() )
		end

		if ( error )
			gs_artist = Strings.google_search_quote( artist )
			fl_artist_url = Strings.build_google_feeling_lucky_url( "artist \"albums on jamendo\" #{gs_artist}", "#{site_host()}/en" )
			if ( log?() )
				log( " - built feeling lucky artist url: #{fl_artist_url}" )
				log( "Fetching artist page... ", 0 )
			end

			response, fl_artist_url = HTTP.fetch_page_get( fl_artist_url )
			error = ! response || ! response.body()

			if ( log?() )
				if ( error )
					log( "ERROR" )
				else
					md = /<title>Jamendo : (.*)<\/title>/.match( response.body() )
					error = md == nil || (Strings.normalize( md[1] ) != Strings.normalize( artist ))
					if ( error )
						log( "INVALID PAGE" )
					else
						log( "OK" )
					end
					log( response.body(), 2 ) if ( long_log?() )
				end
			end

		end

		return suggestions if ( error )

		artist_body = response.body()

		log( "Parsing artist album pages...", 0 ) if ( log?() )

		# artist_body should contain the artist page, we have to iterate through the album pages excerpting the lyrics links
		artist_body.split( /<div class='g_album_name'>/ ).each() do |album_entry|
			if ( md = /<a href="(\/en\/album\/[0-9]+)" >([^<]+)<\/a>/.match( album_entry ) )

				album_url = "http://#{site_host()}#{md[1]}"
				album_name = md[2]

				log( "\n - #{album_url}", 0 ) if ( log?() )

				response, album_url = HTTP.fetch_page_get( album_url )
				next if ( ! response || ! response.body() )

				album_body = response.body()
				album_body.tr_s!( " \n\r\t", " " )

				album_body.sub!( /^.*<div class="block_heavy_titleright">/, "" )
				album_body.sub!( /<\/tbody>.*$/, "" )

				# album_body contains the album page, we have to extract the lyrics links
				album_body.split( "<td class=\"title_tracks\">" ).each() do |song_entry|
					if ( md = /<a href='(\/en\/track\/[0-9]+)'> ([^<]+) <\/a>/.match( song_entry ) )
						suggestions << { "url"=>"http://#{site_host()}#{md[1]}", "artist"=>artist, "title"=>md[2].strip() }
					end
				end

			end
		end

		if ( log?() )
			log( suggestions.empty?() ? " NO SUGGESTIONS FOUND" : "" )
			suggestions.each(){ |sugg| log( " - art: #{sugg["artist"]} | tit: #{sugg["title"]} | url: #{sugg["url"]}" ) }
		end

		return suggestions
	end

	def build_lyrics_fetch_data( artist, title, album=nil, year=nil )
		url =  "http://#{site_host()}/en/get/track/list/track-artist-album/lyricstext/plain/?"
		url << "searchterm=#{CGI.escape(title)}"
		url << "&artist_searchterm=#{CGI.escape(artist)}"
		url << "&album_searchterm=#{CGI.escape(album)}" if ( album )
		return { "url" => url }
	end

	def parse_lyrics( url, body, artist, title, album=nil, year=nil )

		body.tr_s!( " \n\r\t", " " )

		lyrics_data = {}

		if ( /track-artist-album/.match( url ) )

			md = /[?&]searchterm=([^&]+)(&?|$)/.match( url )
			lyrics_data["title"] = CGI.unescape( md[1] ) if ( md )

			md = /[?&]artist_searchterm=([^&]+)(&?|$)/.match( url )
			lyrics_data["artist"] = CGI.unescape( md[1] ) if ( md )

			md = /[?&]album_searchterm=([^&]+)(&?|$)/.match( url )
			lyrics_data["album"] = CGI.unescape( md[1] ) if ( md )

			lyrics_data["lyrics"] = body if ( ! body.empty?() )

		else

			return lyrics_data if ( ! body.sub!( /^.*<div id='page_content_inner'>/, "" ) )

			if ( md = /<div><div><b>([^-]+) - ([^(]+) \(([0-9]+)\) ([^<]+)<\/b><\/div><div>/.match( body ) )
				tags = ["artist", "album", "track", "title"]
				tags.each() { |tag| lyrics_data[tag] = md[tags.index( tag ) + 1] }
			end

			return lyrics_data if ( ! body.sub!( /^.*<\/div><div>/, "" ) )
			return lyrics_data if ( ! body.sub!( /<\/div>.*$/, "" ) )

			body.gsub!( /\ ?<br ?\/?> ?/i, "\n" )

			lyrics_data["lyrics"] = body

		end

		return lyrics_data
	end

end
