# Copyright (C) 2006-2008 by Sergio Pistone
#  sergio_pistone@yahoo.com.ar
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the
#  Free Software Foundation, Inc.,
#  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

$LOAD_PATH << File.expand_path(File.dirname(__FILE__))

require "utils/strings"
require "utils/htmlentities"
require "lyrics"

require "cgi"

class LeosLyrics < Lyrics

	def LeosLyrics.site_host()
		return "www.leoslyrics.com"
	end

	def LeosLyrics.site_name()
		return "Leos Lyrics"
	end

	def LeosLyrics.get_test_data()
		return [
			{ "artist"=>"Radiohead", "album"=>"Kid A", "title"=>"Optimistic" },
			{ "artist"=>"Nirvana", "album"=>"In Utero", "title"=>"All Apologies" },
			{ "artist"=>"System Of A Down", "album"=>"Toxicity", "title"=>"Chop Suey" },
			{ "artist"=>"A Perfect Circle", "album"=>"Thirteen Step", "title"=>"The Noose" },
		]
	end

	def LeosLyrics.build_song_add_url( artist, title, album=nil, year=nil )
		add_url =  "http://#{site_host()}/submit.php?artist=#{CGI.escape( artist )}&song=#{CGI.escape( title )}"
		add_url << "&album=#{CGI.escape( album )}" if ( album )
		return add_url
	end

	def LeosLyrics.build_google_feeling_lucky_url( artist, title=nil )
		query = Strings.google_search_quote( title ? "#{artist} - #{title} lyrics" : "#{artist} lyrics" )
		return Strings.build_google_feeling_lucky_url( query, self.site_host() )
	end

	def build_lyrics_fetch_data( artist, title, album=nil, year=nil )
		return { "url" => build_google_feeling_lucky_url( artist, title ) }
	end

	def check_lyrics_page( page_body, artist, title, album=nil, year=nil )
		md = /<TITLE>([^<]+)-\s*([^<]+)\s*lyrics\s*<\/TITLE>/im.match( page_body )
		return false if ! md
		return	Strings.normalize( artist ) == Strings.normalize( md[1] ) &&
				Strings.normalize( title ) == Strings.normalize( md[2] )
	end

	def parse_lyrics( url, body, artist, title, album=nil, year=nil )

		body = Strings.latin12utf8( body )
		body.tr_s!( " \n\r\t", " " )
		HTMLEntities.decode!( body )

		lyrics_data = {}

		if ( md = /<TITLE> ?(.+) ?- ?(.+) ?lyrics ?<\/TITLE>/.match( body ) )
			lyrics_data["artist"], lyrics_data["title"] = md[1].strip(), md[2].strip()
		end

		if ( md = /<font face="[^"]+" size=-1>(.+)<\/font>/.match( body ) )
			lyrics_data["lyrics"] = md[1]
			lyrics_data["lyrics"].gsub!( /<\/font>.*/, "" )
			lyrics_data["lyrics"].gsub!( /\ ?<br ?\/?> ?/i, "\n" )
			lyrics_data["lyrics"].gsub!( /\n{3,}/, "\n\n" )
		end

		return lyrics_data
	end

	# site's search is currently disabled
	# def build_suggestions_fetch_data( artist, title, album=nil, year=nil )
	# 	return	{ "url" =>	"http://#{site_host()}/advanced.php?" \
	# 						"artistmode=1&artist=#{CGI.escape( Strings.utf82latin1( artist ) )}&" \
	# 						"songmode=1&song=#{CGI.escape( Strings.utf82latin1( title ) )}&mode=0" }
	# end

	def build_suggestions_fetch_data( artist, title, album=nil, year=nil )
		return { "url" => build_google_feeling_lucky_url( artist ) }
	end

	def check_suggestions_page( page_body, artist, title, album=nil, year=nil )
		md = /<TITLE>\s*Leo's Lyrics Database\s*-\s*([^<]+)\s*lyrics\s*<\/TITLE>/im.match( page_body )
		return md ? Strings.normalize( artist ) == Strings.normalize( md[1] ) : false
	end

	def parse_suggestions( url, body, artist, title, album=nil, year=nil )

		body = Strings.latin12utf8( body )
		body.tr_s!( " \n\r\t", " " )
		HTMLEntities.decode!( body )

		suggestions = []

		md = /<ul>(.*)<\/ul>/i.match( body )
		return suggestions if ( ! md )

		md[1].split( "<li>" ).each do |entry|
			if ( md = /<a href="(\/listlyrics.php;jsessionid=[^"]+)">([^<]+)<\/a>/.match( entry ) )
				suggestions << { "url"=>"http://#{site_host()}#{md[1]}", "artist"=>artist, "title"=>md[2] }
			end
		end

		return suggestions
	end

end
