# Copyright (C) 2006-2008 by Sergio Pistone
#  sergio_pistone@yahoo.com.ar
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the
#  Free Software Foundation, Inc.,
#  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

$LOAD_PATH << File.expand_path(File.dirname(__FILE__))

require "utils/strings"
require "utils/http"
require "gui/gui"
require "wiki_lyrics"

require "cgi"
require "uri"

class LyricWiki < WikiLyrics

	def LyricWiki.site_host()
		return "lyricwiki.org"
	end

	def LyricWiki.site_name()
		return "LyricWiki"
	end

	def LyricWiki.control_page()
		return "LyricWiki:Wiki-Lyrics"
	end

	def parse_lyrics( url, body, artist, title, album=nil, year=nil )

		body.gsub!( /â€™|�/, "'" ) # replace bizarre characters with apostrophes

		lyrics_data = { "custom" => {} }

		lyrics_data["custom"]["autogen"] = (/\[\[Category:[Rr]eview[_ ]Me\]\]/.match( body ) != nil)

		# search album, year and artist information
		if ( md = /\{\{\s*[Ss]ong\s*\|.*$/.match( body ) )
			unnamed_params_names = { 1 => "album_and_year", 2 => "artist" }
			template_data = parse_template( md[0] )
			template_data["params"].each() do |key, value|
				lyrics_data["custom"][key] = value if ( (key = unnamed_params_names[key]) && value.is_a?( String ) )
			end
		elsif ( md = /On (''')?\[\[[^\|]+\|([^\|]+)\]\](''')? by (''')?\[\[([^\]]+)\]\](''')?/.match( body ) )
			lyrics_data["custom"]["album_and_year"] = md[2]
			lyrics_data["custom"]["artist"] = md[5]
		end

		if ( lyrics_data["custom"].include?( "album_and_year" ) )
			if ( md = /^(.+) \(([\?0-9]{4,4})\)$/.match( lyrics_data["custom"]["album_and_year"] ) )
				lyrics_data["album"] = md[1]
				lyrics_data["year"] = md[2] if ( md[2].to_i() > 1900 )
			end
		end

		# search title information (and other information that hasn't been found yet)
		if ( md = /\{\{\s*[Ss]ongFooter\s*\|.*$/.match( body ) )
			template_data = parse_template( md[0] )
			template_data["params"].each() do |key, value|
				lyrics_data["custom"][key.to_s()] = value if ( value.is_a?( String ) )
			end
		elsif ( md = /\[[^\s\]]+ ([^\]]+)\] on Amazon$/.match( body ) )
			lyrics_data["custom"]["song"] = md[1].strip()
		end

		if ( md = /\*?\s*Composer: *([^\n]+)/.match( body ) )
			lyrics_data["custom"]["credits"] = md[1].strip()
		end
		if ( md = /\*?\s*Lyrics by: *([^\n]+)/.match( body ) )
			lyrics_data["custom"]["lyricist"] = md[1].strip()
		end

		lyrics_data["artist"] = lyrics_data["custom"]["artist"] if ( lyrics_data["custom"].include?( "artist" ) )
		lyrics_data["title"] = lyrics_data["custom"]["song"] if ( lyrics_data["custom"].include?( "song" ) )
		lyrics_data["album"] = lyrics_data["custom"]["album"] if ( lyrics_data["custom"].include?( "album" ) )

		if ( md = /<lyrics?>(.*)<\/lyrics?>/im.match( body ) )
			body = md[1]
			if ( /\s*\{\{[Ii]nstrumental\}\}\s*/.match( body ) )
				body = "<tt>(Instrumental)</tt>"
			else
				body.gsub!( /[ \t]*[\r\n][ \t]*/m, "\n" )
			end
		else
			if ( /\s*\{\{[Ii]nstrumental\}\}\s*/.match( body ) )
				body = "<tt>(Instrumental)</tt>"
			else
				body.gsub!( /\{\{.*\}\}\n?/, "" )
				body.gsub!( /\[\[Category:.*\]\]\n?/, "" )
				body.gsub!( /On '''\[\[.*?\n/i, "" )
				body.gsub!( /By '''\[\[.*?\n/i, "" )
				body.gsub!( /\ *== *Credits *==.*$/im, "" )
				body.gsub!( /\ *== *(External *Links|Links) *==.*$/im, "" )
				body = body.split( "\n" ).collect() do |line|
					if ( line.index( /\s/ ) == 0 )
						"\n" + line
					else
						line
					end
				end.join( "" )
				body.gsub!( /\s*<br ?\/?>\s*/i, "\n" )
			end
		end

		if ( Strings.empty?( body ) )
			return lyrics_data
		else
			# expand ruby tags:
			body.gsub!( /\{\{ruby\|([^\|]*)\|([^\}]*)\}\}/, "<ruby><rb>\1</rb><rp>(</rp><rt>\2</rt><rp>)</rp></ruby>" )
			# take care of multiple lyrics tags:
			body.gsub!( /(\{\|\s*\|-\s*\||\|\|)\s*==\s*([^=]+)\s*==/, "<br/><b>\2</b>" )
			body.gsub!( /\s*==\s*([^=]+)\s*==/, "\n<br/><b>\\1</b>" )
			body.gsub!( /<\/?lyric>/i, "" )
			lyrics_data["lyrics"] = body
			lyrics_data["add_url"] = "#{url}&action=edit"
			return lyrics_data
		end
	end

	def LyricWiki.build_tracks( tracks )
		tracks_data = parse_tracks( tracks )
		album_artist = cleanup_title_token( tracks_data["artist"] )
		ret = ""
		tracks.each() do |track|
			track_artist = cleanup_title_token( track["artist"] )
			track_title  = cleanup_title_token( track["title"] )
			tc_track_title = Strings.titlecase( track_title )
			# artist_param = (album_artist == "(Various Artists)" || album_artist == "Various Artists") ? "by" : "artist"
			# ret += "# {{track|title=#{track_title}|#{artist_param}=#{track_artist}}}\n"
			# ret += "# {{track|title=#{track_title}|#{artist_param}=#{track_artist}|display=#{tc_track_title}}}\n"
			ret += "# '''[[#{track_artist}:#{track_title}|#{tc_track_title}]]'''\n"
		end
		return ret
	end

	def LyricWiki.build_album_page( autogen, artist, album, year, month, day, tracks, album_art )

		raise ArgumentError if ( Strings.empty?( artist ) || Strings.empty?( album ) || Strings.empty?( tracks ) )

		f_letter = get_first_letter( album )

		contents = \
		"#{autogen.to_s() != "false" ? "[[Category:Review Me]]\n" : ""}" \
		"{{Album\n" \
		"|Artist   = #{artist}\n" \
		"|Album    = #{album}\n" \
		"|fLetter  = #{f_letter}\n" \
		"|Released = #{build_date( year, month, day )}\n" \
		"|Cover    = #{album_art}\n" \
		"}}\n"
		# "|Genre    = #{genre}\n" \
		# "|Length   = #{length}\n" \

		return \
		"#{contents}\n" \
		"#{tracks.strip()}\n" \
		"\n" \
		"{{AlbumFooter\n" \
		"|artist=#{artist}\n" \
		"|album=#{album}\n" \
		"}}\n"
	end

	def LyricWiki.build_song_page( autogen, artist, title, album, year, credits, lyricist, lyrics )

		raise ArgumentError if ( artist == nil || title == nil )

		f_letter = get_first_letter( title )
		year = year.to_i() > 1900 ? year.to_s() : "????"

		if ( lyrics )
			lyrics = lyrics.gsub( /<ruby><rb>([^<]*)<\/rb><rp>\(<\/rp><rt>([^<]*)<\/rt><rp>\)<\/rp><\/ruby>/, "{{ruby|\1|\2}}" )
		end

		return \
		"#{autogen.to_s() != "false" ? "[[Category:Review Me]]\n" : ""}" \
		"{{Song|#{Strings.empty?( album ) ? "" : "#{album} (#{year})"}|#{artist}}}\n\n" \
		"<lyric>\n#{Strings.empty?( lyrics ) ? "{{instrumental}}" : lyrics}\n</lyric>\n\n" \
		"#{Strings.empty?( credits ) && Strings.empty?( lyricist ) ? "" : "==Credits==\n"}" \
		"#{Strings.empty?( credits ) ? "" : "*Composer: #{credits}\n"}" \
		"#{Strings.empty?( lyricist ) ? "" : "*Lyrics by: #{lyricist}\n"}" \
		"\n" \
		"{{SongFooter\n" \
		"|artist=#{artist}\n" \
		"|song=#{title}\n" \
		"|fLetter=#{f_letter}\n" \
		"}}\n"

	end

	def LyricWiki.build_album_art_name( artist, album, year, extension="jpg", cleanup=true )
		if ( cleanup )
			artist = cleanup_title_token( artist )
			album = cleanup_title_token( album )
		end
		album_art_name = "#{artist} - #{album}#{Strings.empty?( extension ) ? "" : ".#{extension.strip()}"}".gsub( " ", "_" )
		return Strings.remove_invalid_filename_chars( album_art_name )
	end

	def LyricWiki.build_album_art_description( artist, album, year, cleanup=true )
		if ( cleanup )
			artist = cleanup_title_token( artist )
			album = cleanup_title_token( album )
		end
		return \
		"{{Albumcover/Upload|\n" \
		"|artist = #{artist}\n" \
		"|album  = #{album}\n" \
		"|year   = #{year}\n" \
		"}}\n"
	end

	def LyricWiki.find_album_art_name( artist, album, year )

		normalized_artist = cleanup_title_token( artist )
		Strings.remove_invalid_filename_chars!( normalized_artist )
		Strings.normalize!( normalized_artist )
		normalized_artist.gsub!( " ", "" )

		normalized_album = cleanup_title_token( album )
		Strings.remove_invalid_filename_chars!( normalized_album )
		Strings.normalize!( normalized_album )
		normalized_album.gsub!( " ", "" )

		artist = cleanup_title_token( artist )
		Strings.remove_invalid_filename_chars!( artist )
		search_url = "http://#{site_host()}/index.php?ns6=1&search=#{CGI.escape( artist )}&searchx=Search&limit=500"
		response, search_url = HTTP.fetch_page_get( search_url )

		return nil if ( response == nil || response.body() == nil )

		candidates = []
		parse_search_results( response.body(), true ).each() do |result|

			next if ( result["title"].index( "Image:" ) != 0 )

			normalized_title = Strings.normalize( result["title"] )
			normalized_title.gsub!( " ", "" )

			matches = 0
			idx1 = normalized_title.index( normalized_artist )
			matches += 1 if ( idx1 )
			idx1 = idx1 ? idx1 + normalized_artist.size() : 0
			idx2 = normalized_title.index( normalized_album, idx1 )
			matches += 2 if ( idx2 )

			candidates.insert( -1, [ matches, result["title"] ] ) if ( matches > 1 )
		end

		if ( candidates.size > 0 )
			candidates.sort!() { |x,y| y[0] <=> x[0] }
			return URI.decode( candidates[0][1].slice( "Image:".size()..-1 ).gsub( " ", "_" ) )
		else
			return nil
		end
	end

	def LyricWiki.get_first_letter( title )
		if ( title.index( /^[a-zA-Z]/ ) == 0 )
			return title.slice( 0, 1 )
		elsif ( title.index( /^[0-9]/ ) == 0 )
			return "0-9"
		else
			return "Symbol"
		end
	end

	def LyricWiki.cleanup_title_token!( title, downcase=false )
		title.gsub!( /\[[^\]\[]*\]/, "" )
		title.gsub!( /[\[|\]].*$/, "" )
		title.gsub!( /`|´|’/, "'" )
		title.gsub!( /''|«|»/, "\"" )
		title.squeeze!( " " )
		title.strip!()
		title.gsub!( "+", "and" )
		Strings.titlecase!( title, false, downcase )
		return title
	end

end
