#  Copyright (C) 2007 by Sergio Pistone
#  sergio_pistone@yahoo.com.ar
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the
#  Free Software Foundation, Inc.,
#  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

$LOAD_PATH << File.expand_path(File.dirname(__FILE__))

require "utils/strings"
require "utils/htmlentities"
require "lyrics"

class LyricsDownload < Lyrics

	@@white_chars = "'\"¿?¡!()[].,;:-/& "

	def LyricsDownload.site_host()
		return "www.lyricsdownload.com"
	end

	def LyricsDownload.site_name()
		return "Lyrics Download"
	end

	def LyricsDownload.get_test_data()
		return [
			{ "artist"=>"Nirvana", "album"=>"Nevermind", "title"=>"Smells Like Teen Spirit" },
			{ "artist"=>"Radiohead", "album"=>"Kid A", "title"=>"Optimistic" },
			{ "artist"=>"Massive Attack", "album"=>"Protection", "title"=>"Protection" },
			{ "artist"=>"Portishead", "album"=>"Dummy", "title"=>"Wandering Star" },
		]
	end

	def cleanup_artist( artist )
		artist = artist.gsub( /^the /i, "" )
		Strings.remove_vocal_accents!( artist )
		artist.gsub!( "&", "and" )
		artist.tr!( @@white_chars, " " )
		artist.strip!()
		artist.tr!( " ", "-" )
		return artist
	end

	def cleanup_title( title )
		title = Strings.remove_vocal_accents( title )
		title.gsub!( "&", "and" )
		title.tr!( @@white_chars, " " )
		title.strip!()
		title.tr!( " ", "-" )
		return title
	end

	def build_lyrics_fetch_data( artist, title, album=nil, year=nil )
		return { "url" => "http://#{site_host()}/#{cleanup_artist( artist )}-#{cleanup_title( title )}-lyrics.html" }
	end

	def parse_lyrics( url, body, artist, title, album=nil, year=nil )

		body = Strings.latin12utf8( body )
		body.tr_s!( " \n\r\t", " " )
		body.tr_s!( "", "'" )
# 		body.tr_s!( "‘", "'" )

		lyrics_data = {}

		if ( md = /<p align="center">\. :<a id="[^"]+ lyrics" class="style20" href="[^"]+-lyrics.html">([^<]+)<\/a> - ([^<]+) : \.<\/td>/.match( body ) )
			lyrics_data["artist"], lyrics_data["title"] = md[1].strip(), md[2].strip()
		end

		return lyrics_data if ( ! body.gsub!( /^.*<center> ?<font class="txt_1">/, "" ) )
		return lyrics_data if ( ! body.gsub!( /<\/font> ?<\/center>.*$/, "" ) )

		lyrics_data["lyrics"] = body
		lyrics_data["lyrics"].gsub!( /\ ?<br ?\/?> ?/i, "\n" )
		lyrics_data["lyrics"].strip!()

		return lyrics_data
	end

	def build_suggestions_fetch_data( artist, title, album=nil, year=nil )
		return { "url" => "http://#{site_host()}/#{cleanup_artist( artist )}-lyrics.html" }
	end

	def parse_suggestions( url, body, artist, title, album=nil, year=nil )

		body = Strings.latin12utf8( body )
		body.tr_s!( " \n\r\t", " " )
		body.tr_s!( "", "'" )

		suggestions = []

		return suggestions if ( ! body.sub!( /^.*Lyrics list aplhabetically:<\/font><\/td>/, "" ) )
		return suggestions if ( ! body.sub!( /<\/ul> ?<\/td> ?<\/tr> ?<\/table> ?<center><div>.*$/, "" ) )

		body.split( "</li>" ).each() do |entry|
			if ( md = /<a class="txt_1" href="([^"]+)"><font size=2>([^<]+) Lyrics<\/font><\/a>/.match( entry ) )
				suggestions << { "url"=>"http://#{site_host()}/#{md[1]}", "artist"=>artist, "title"=>md[2] }
			end
		end

		return suggestions
	end

end
