#  Copyright (C) 2007 by Sergio Pistone
#  sergio_pistone@yahoo.com.ar
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the
#  Free Software Foundation, Inc.,
#  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

$LOAD_PATH << File.expand_path(File.dirname(__FILE__))

require "utils/strings"
require "lyrics"

class SeekLyrics < Lyrics

	@@white_chars = "'\"¿?¡!()[].,;:-/& "

	def SeekLyrics.site_host()
		return "www.seeklyrics.com"
	end

	def SeekLyrics.site_name()
		return "Seek Lyrics"
	end

	def SeekLyrics.get_test_data()
		return [
			{ "artist"=>"Nirvana", "album"=>"Nevermind", "title"=>"Smells Like Teen Spirit" },
			{ "artist"=>"Radiohead", "album"=>"Kid A", "title"=>"Optimistic" },
			{ "artist"=>"Massive Attack", "album"=>"Protection", "title"=>"Protection" },
			{ "artist"=>"Portishead", "album"=>"Dummy", "title"=>"Wandering Star" },
		]
	end

	def SeekLyrics.build_song_add_url( artist, title, album=nil, year=nil )
		return "http://#{site_host()}/submit.php"
	end

	def cleanup_artist( artist )
		artist = artist.gsub( /^the /i, "" )
		artist.tr( @@white_chars, " " )
		artist.strip!()
		artist.tr!( " ", "-" )
		return artist
	end

	def cleanup_title( title )
		title = title.tr( @@white_chars, " " )
		title.strip!()
		title.tr!( " ", "-" )
		return title
	end

	def build_lyrics_fetch_data( artist, title, album=nil, year=nil )
		return { "url" => "http://#{site_host()}/lyrics/#{cleanup_artist( artist )}/#{cleanup_title( title )}.html", "headers" => { "Cookie"=>"pass=deleted" } }
	end

	def fetch_lyrics_page( fetch_data )
		return nil, nil if ( fetch_data["url"] == nil )
		fetch_data.merge!( { "headers" => { "Cookie"=>"pass=deleted" } } )
		return super( fetch_data )
	end

	def parse_lyrics( url, body, artist, title, album=nil, year=nil )

		body = Strings.latin12utf8( body )
		body.tr_s!( " \n\r\t", " " )
		body.tr_s!( "", "'" )

		lyrics_data = {}

		if ( md = /<b><h2>([^<-]+) - ([^<]+) Lyrics<\/h2><\/b>/i.match( body ) )
			lyrics_data["artist"] = md[1].strip()
			lyrics_data["title"]  = md[2].strip()
		end

		return lyrics_data if ( ! body.sub!( /^(.*)<a href="http:\/\/www\.ringtonematcher\.com.*$/i, "\\1" ) )
		return lyrics_data if ( ! body.sub!( /^.*<img src="\/images\/phone-right\.gif" [^<]+><\/a>/i, "" ) )

		lyrics_data["lyrics"] = body
		lyrics_data["lyrics"].gsub!( /\ ?<br ?\/?> ?/i, "\n" )
		lyrics_data["lyrics"].strip!()

		return lyrics_data
	end

	def build_suggestions_fetch_data( artist, title, album=nil, year=nil )
		return { "url" => "http://#{site_host()}/lyrics/#{cleanup_artist( artist )}/showall.html" }
	end

	def fetch_suggestions_page( fetch_data )
		return nil, nil if ( fetch_data["url"] == nil )
		fetch_data.merge!( { "headers" => { "Cookie"=>"pass=deleted" } } )
		return super( fetch_data )
	end

	def parse_suggestions( url, body, artist, title, album=nil, year=nil )

		body = Strings.latin12utf8( body )
		body.tr_s!( " \n\r\t", " " )
		body.tr_s!( "", "'" )

		suggestions = []

		return suggestions if ( ! body.gsub!( /.*<tr><td width="50%"><\/td><td width="50%"><\/td><\/tr>/, "" ) )
		return suggestions if ( ! body.gsub!( /<\/table>.*$/, "" ) )

		body.split( /<td>&nbsp;-&nbsp;&nbsp;/i ).each() do |entry|
			if ( md = /<a href="([^"]+)"[^>]*>([^<]+)<\/a><\/td>/i.match( entry ) )
				suggestions << { "url"=>"http://#{site_host()}#{md[1]}", "artist"=>artist, "title"=>md[2] }
			end
		end

		return suggestions
	end

end
