# Copyright (C) 2006-2008 by Sergio Pistone
#  sergio_pistone@yahoo.com.ar
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the
#  Free Software Foundation, Inc.,
#  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

$LOAD_PATH << File.expand_path(File.dirname(__FILE__))

require "utils/strings"
require "utils/htmlentities"
require "lyrics"

require "cgi"

class TerraLetras < Lyrics

	def TerraLetras.site_host()
		return "letras.terra.com.br"
	end

	def TerraLetras.site_name()
		return "Terra Letras"
	end

	def TerraLetras.known_url?( url )
		return url.index( "http://#{site_host}" ) == 0 || /^http:\/\/.*\.letras\.terra\.com\.br\/letras\/[0-9]+/.match( url )
	end

	def TerraLetras.get_test_data()
		return [
			{ "artist"=>"The Cranberries", "album"=>"Everybody Else Is Doing It, So Why Can't We?", "title"=>"Linger" },
			{ "artist"=>"Radiohead", "album"=>"Kid A", "title"=>"Optimistic" },
			{ "artist"=>"Mark Lanegan", "album"=>"Field Songs", "title"=>"One Way Street" },
			{ "artist"=>"The Smashing Pumpkins", "album"=>"Siamese Dream", "title"=>"Mayonaise" },
		]
	end

	def TerraLetras.build_song_add_url( artist, title, album=nil, year=nil )
		return "http://#{site_host()}/envie_artista.php"
	end

	def TerraLetras.build_google_feeling_lucky_url( artist, title=nil )
		if ( title )
			query = Strings.google_search_quote( "#{title} letra" )
			query << " "
			query << Strings.google_search_quote( artist )
		else
			query = Strings.google_search_quote( "letras de #{artist}" )
			query << " "
			query << Strings.google_search_quote( "#{artist} letras" )
		end
		return Strings.build_google_feeling_lucky_url( query, site_host() )
	end

	def build_lyrics_fetch_data( artist, title, album=nil, year=nil )
		artist = Strings.utf82latin1( artist )
		title  = Strings.utf82latin1( title )
		return { "url" => "http://#{site_host()}/winamp.php?musica=#{CGI.escape(title)}&artista=#{CGI.escape(artist)}" }
	end

	def parse_lyrics( url, body, artist, title, album=nil, year=nil )

		body = Strings.latin12utf8( body )
		body.tr_s!( " \n\r\t", " " )

		lyrics_data = {}

		if ( url.index( "http://#{site_host}/winamp.php?" ) == 0 ) # direct fetch
			if ( md = /<h1><a href='[^']+' target='_blank'>([^<]+)<\/a><\/h1>/.match( body ) )
				lyrics_data["title"] = md[1]
			end
			if ( md = /<h2><a href='[^']+' target='_blank'>([^<]+)<\/a><\/h2>/.match( body ) )
				lyrics_data["artist"] = md[1]
			end
			return lyrics_data if ( ! body.gsub!( /^.*corrigir letra<\/a><\/p><p>/, "" ) )
			return lyrics_data if ( ! body.gsub!( /<\/p>.*$/, "" ) )
		else # fetched from suggestions/url
			if ( md = /<h2>([^<]+)<\/h2> <h2 id='sz'>([^<]+)<\/h2>/.match( body ) )
				lyrics_data["title"], lyrics_data["artist"] = md[1], md[2]
			end
			return lyrics_data if ( ! body.sub!( /^.*<p id='cmp'>[^<]*<\/p> <p>/, "" ) )
			return lyrics_data if ( ! body.sub!( /<\/p>.*/, "" ) )
		end

		lyrics_data["lyrics"] = body
		lyrics_data["lyrics"].gsub!( /\ ?<br ?\/?> ?/i, "\n" )
		lyrics_data["lyrics"].gsub!( /\n{3,}/, "\n\n" )

		return lyrics_data
	end

	def build_suggestions_fetch_data( artist, title, album=nil, year=nil )
		return { "url" => build_google_feeling_lucky_url( artist ) }
	end

	def parse_suggestions( url, body, artist, title, album=nil, year=nil )

		body = Strings.latin12utf8( body )
		body.tr_s!( " \n\r\t", " " )
		HTMLEntities.decode!( body )

		suggestions = []

		return suggestions if ( ! body.gsub!( /^.*<ul class='top' id='bgn'>/, "" ) )
		return suggestions if ( ! body.gsub!( /<\/ul>.*$/, "" ) )

		body.split( /<\/li> ?<li>/ ).each do |entry|
			if ( md = /<a href="([^"]+)">([^<]+) - ([^<]+)<\/a>/.match( entry ) )
				suggestions << { "url"=>"http://#{site_host()}#{md[1]}", "artist"=>md[2], "title"=>md[3] }
			end
		end

		return suggestions
	end

end
