# Copyright (C) 2006-2008 by Sergio Pistone
#  sergio_pistone@yahoo.com.ar
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the
#  Free Software Foundation, Inc.,
#  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

require "rexml/document"

module XMLHash

	def XMLHash.write( filename, hash )

		begin
			file = File.new( filename )
			xml = REXML::Document.new( file )
			file.close()
			root = xml.root
		rescue Exception
			xml = REXML::Document.new( "<?xml version='1.0' encoding='UTF-8' ?>" )
			root = xml.add_element( "settings" )
		end

		begin
			file = File.new( filename, File::CREAT|File::TRUNC|File::RDWR, 0644 )
			hash.each do | key, value |
				if ( root.elements[key] )
					root.elements[key].text = value.to_s()
				else
					root.add_element( key ).text = value.to_s()
				end
			end
			xml.write( file )
			file.close()
			return true
		rescue Errno::ENOENT
			return false
		end

	end

	def XMLHash.read( filename, hash )

		begin
			file = File.new( filename )
			elements = REXML::Document.new( file ).root.elements
			file.close()
			return false if ( elements == nil )
		rescue Exception
			return false
		end

		missing_keys = false

		keys = hash.clone()
		keys.each do |key, val|
			if ( elements[key] )
				hash[key] = elements[key].text()
				hash[key] = "" if ( hash[key] == nil )
			else
				missing_keys = true
			end
		end

		return ! missing_keys

	end

end

